/* 
** widgets.c: Widget manipulation functions
** copyright: (c) 2003 by Lszl Pere
** email: pipas@linux.pte.hu
**
** This program is free software; you can redistribute it and/or 
** modify  it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of
** the License, or (at your option) any later version.
**
** $Id: widgets.c,v 1.2 2004/11/25 19:11:12 pipas Exp pipas $
** $Log: widgets.c,v $
** Revision 1.2  2004/11/25 19:11:12  pipas
** Small enhancements.
**
** Revision 1.1  2004/11/18 16:47:05  root
** Initial revision
**
 */

#include "widgets.h"
#include "stringman.h"
char *CHOOSERCONT=NULL;


static void fill_list_by_items(AttributeSet *Attr, GtkWidget * list) {
	GtkWidget *item;
	char      *text;
	g_assert(Attr != NULL && list != NULL);
	
	text = attributeset_get_first(Attr, ATTR_ITEM);
	if (text == NULL)
		return;
	
	while (text != NULL) {
		item = gtk_list_item_new_with_label(text);
		gtk_object_set_user_data(GTK_OBJECT(item), (gpointer) text);
		gtk_container_add(GTK_CONTAINER(list), item);
		text = attributeset_get_next(Attr, ATTR_ITEM);
	}
}

void fill_clist_by_items(AttributeSet *Attr, GtkWidget *list, int separator) {
	char *text;
	list_t *sliced;

	g_assert(Attr != NULL && list != NULL);
	
	text = attributeset_get_first(Attr, ATTR_ITEM);
	if (text == NULL)
		return;
	
	while (text != NULL){
		sliced = linecutter(text, separator);
		gtk_clist_append(GTK_CLIST(list), sliced->line);
		text = attributeset_get_next(Attr, ATTR_ITEM);
	}
}

static void fill_combo_by_items(AttributeSet *Attr, GtkWidget *combo) {
	GList *glist = NULL;
	char *text;
	
	g_assert(Attr != NULL && combo != NULL);
	text = attributeset_get_first(Attr, ATTR_ITEM);
	if (text == NULL)
		return;
	///GN: added default - setting it to be first item in list...
	if (attributeset_is_avail(Attr, ATTR_DEFAULT)) {
		glist = g_list_append(glist,
			attributeset_get_first(Attr, ATTR_DEFAULT));
	}
	while (text != NULL) {
		glist = g_list_append(glist, text);
		text = attributeset_get_next(Attr, ATTR_ITEM);
	}
	
	gtk_combo_set_popdown_strings(GTK_COMBO(combo), glist);
}

char *widget_get_text_value(GtkWidget * widget, int type) {
	GList *item;
	GtkAdjustment *adj;

	if (widget == NULL) {
		fprintf(stderr, "%s(): NULL Widget.\n", __func__);
		fflush(stderr);
		return ("NULL WIDGET");
	}

	switch (type) {
		
		case WIDGET_CHOOSER:
#ifdef USE_GTK2
			return gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
#else
			return CHOOSERCONT;
#endif
			break;
		case WIDGET_ENTRY:
			return (char *) gtk_entry_get_text(GTK_ENTRY(widget));
			break;
		///20130514: Missing getting variable value from edit
		case WIDGET_EDIT:
#ifdef USE_GTK2
			{
			GtkTextBuffer *buffer;
			GtkTextIter start, end;
			buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
			gtk_text_buffer_get_start_iter(buffer, &start);
			gtk_text_buffer_get_end_iter(buffer, &end);
			return (char *) gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
			}
#else
			return (char *) gtk_editable_get_chars( GTK_EDITABLE(widget), 0, -1 );
#endif 
			break;	
		case WIDGET_COMBO:
			return (char *) gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(widget)->entry));
			break;
		case WIDGET_CHECKBOX:
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
				return ("true");
			else
				return ("false");
			break;
		case WIDGET_RADIO:
			if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
				return ("true");
			else
				return ("false");
			break;
		case WIDGET_LIST:
			item = GTK_LIST(widget)->selection;
			if (item != NULL)
				return (gtk_object_get_user_data(item->data));
			else
				return ("");
			break;
		case WIDGET_TABLE:
			/*
		 	** I simply could not find a function to get the selectef item,
		 	** so managed the selected item by callback function.
		 	*/
			return (NULL);
			break;
		case WIDGET_HSCALE:
		case WIDGET_VSCALE:
			{
			gchar            *string;
			gchar             value[32];
			gdouble           val;

			adj = gtk_range_get_adjustment( GTK_RANGE(widget) );
        	if (adj != NULL) {
        		val = (adj->value);
        	}	
		
			sprintf(value, "%f", val);
			string = g_strdup(value);
	
			return string;
			}
			break;
		default:
			return (NULL);
	}
}

void fill_label_by_file(GtkWidget * widget, char *filename) {
	struct stat st;
	char *filebuffer;
	int infile;

	if (stat(filename, &st) != 0) {
		fprintf(stderr, "%s(): Couldn't stat '%s'.\n", __func__, filename);
		return;
	}

	filebuffer = g_malloc(st.st_size);
	infile = open(filename, O_RDONLY);
	if (infile == -1) {
		fprintf(stderr, "%s(): Could't open '%s' for reading.\n", __func__, filename);
		return;
	}

	read(infile, filebuffer, st.st_size);
	close(infile);

	gtk_label_set_text(GTK_LABEL(widget), filebuffer);
	return;
}

static void save_edit_to_file(GtkWidget * widget, char *filename) {
	int outfile;
#ifdef USE_GTK2
	GtkTextBuffer *buffer;
	GtkTextIter start, end;
#endif
	gchar *text;

	// FIXME: this is stupid, I really should correct this
	//
	if (strncasecmp(filename, "File: ", 6) == 0)
		filename += 6;
	if (strncasecmp(filename, "File:", 5) == 0)
		filename += 5;

	outfile = open(filename, O_WRONLY | O_CREAT | O_TRUNC, S_IRWXU);
	if (outfile == -1) {
		fprintf(stderr, "%s(): Could't open '%s' for writing.\n", __func__, filename);
		return;
	}
#ifdef USE_GTK2 
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_get_start_iter(buffer, &start);
	gtk_text_buffer_get_end_iter(buffer, &end);
	text = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
#else
	text=gtk_editable_get_chars( GTK_EDITABLE(widget), 0, -1 ); 
#endif
	write(outfile, text, strlen(text));
	close(outfile);
}

void fill_edit_by_file(GtkWidget * widget, char *filename) {
	struct stat st;
	char *filebuffer;
	int infile;
#ifdef USE_GTK2
	GtkTextBuffer *buffer;
#endif
	char *message;	

	// FIXME: this is stupid, I really should correct this
	if (strncasecmp(filename, "File: ", 6) == 0)
		filename += 6;
	if (strncasecmp(filename, "File:", 5) == 0)
		filename += 5;

	if (stat(filename, &st) != 0) {
		message = g_malloc(strlen(filename) + 128);
		sprintf(message, "Could not stat '%s'.", filename);
		g_print(message);
		g_free(message);
		return;
	}
	
	filebuffer = g_malloc(st.st_size);
	infile = open(filename, O_RDONLY);
	if (infile == -1) {
		message = g_malloc(strlen(filename) + 128);
		sprintf(message, "Could not open '%s' for reading.", filename);
		g_print(message);
		g_free(message);
		return;
	}

	read(infile, filebuffer, st.st_size);
	close(infile);
	
	//GN: I think we must clear content in editor before inserting the text (?)
#ifdef USE_GTK2
	//test
	gtk_text_buffer_set_text(gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(widget)), "", 0);
	//test end
	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_set_text(buffer, filebuffer, st.st_size);
#else
	//test
	gtk_text_set_point( GTK_TEXT(widget), 0 );
	//forward delete
	gtk_text_forward_delete ( GTK_TEXT(widget),
			gtk_text_get_length( GTK_TEXT(widget) ) );
	//end test
	gtk_text_insert( GTK_TEXT(widget), NULL, NULL, NULL, 
	   									filebuffer, st.st_size );
#endif
}

static void widget_hscale_input_by_command(variable *var, char *command) {
	///most code from gtkdialog 0.8.3
	FILE *infile;
	gchar line[512];
	gint count;

	/// Opening pipe for reading
	if (infile = widget_opencommand(command)) {
		/// Just one line
		if (fgets(line, 512, infile)) {
			/// Enforce end of string in case of max chars read
			line[512 - 1] = 0;
			/// Remove the trailing [CR]LFs
			for (count = strlen(line) - 1; count >= 0; count--)
				if (line[count] == 13 || line[count] == 10) line[count] = 0;
			gtk_adjustment_set_value ((GTK_RANGE(var->Widget))->adjustment, atof(line));
		}
		/// Close the file
		pclose(infile);
	} else {
		fprintf(stderr, "%s(): Couldn't open '%s' for reading.\n", __func__, command);
	}
}

int widget_scale_refresh(variable * var) {
	char *act;
	if (var == NULL || var->Attributes == NULL)
		return FALSE;

	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (input_is_shell_command(act)) {	
			widget_hscale_input_by_command(var, act + 8);
		}
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}
	return 0;
}

int widget_label_refresh(variable * var) {
	char *act;

	if (var == NULL || var->Attributes == NULL)
		return FALSE;

	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (strncasecmp(act, "file:", 5) == 0)
			fill_label_by_file(var->Widget, act + 5);
		if (input_is_shell_command(act))
			fill_label_by_command(var->Widget, act + 8);
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}

	return 0;
}

int widget_edit_refresh(variable * var) {
	char *act;
	if (var == NULL || var->Attributes == NULL)
		return FALSE;

	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		fill_edit_by_file(var->Widget, act);
		act = attributeset_get_next(var->Attributes, 
				ATTR_INPUT);
	}
	return 0;
}

void widget_edit_save(variable * var) {
	char *filename;

	g_assert(var != NULL);
	
	filename = attributeset_get_first(var->Attributes, ATTR_OUTPUT);
	if (filename == NULL){
		g_print("Save activated but no output is given. "
			"Trying to save to the input file...");
		goto try_input;
	}

	while (filename != NULL) {
		save_edit_to_file(var->Widget, filename);
		filename = attributeset_get_next(var->Attributes, 
			ATTR_OUTPUT);
	}
	return;

	/// If there is no <output>, we try to use <input> file.
try_input:
	filename = attributeset_get_first(var->Attributes, ATTR_INPUT);
	if (filename == NULL){
		g_print("No input file either.");
		return;
	}
	save_edit_to_file(var->Widget, filename);
}

void widget_entry_refresh(variable * var) {
	char *act;

	if (var == NULL || var->Attributes == NULL)
		return;
		
	/// The <input> tag
	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (strncasecmp(act, "command:", 8) == 0)
			fill_entry_by_command(var->Widget, act + 8);
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}

	if (attributeset_cmp_left
	    (var->Attributes, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(var->Widget, FALSE);

}

void widget_checkbox_refresh(variable * var) {
	char *act;

	if (var == NULL || var->Attributes == NULL)
		return;
		
	/// The <input> tag
	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (strncasecmp(act, "command:", 8) == 0)
			fill_checkbox_by_command(var->Widget, act + 8);
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}

	if (attributeset_cmp_left
	    (var->Attributes, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(var->Widget, FALSE);

}

void widget_list_refresh(variable * var) {
	char *act;

	if (var == NULL || var->Attributes == NULL)
		return;

	/// The <input> tag
	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (strncasecmp(act, "command:", 8) == 0) {
			fill_list_by_command(var->Widget, act + 8);
		}
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}
	
	/// The <item> tags
	if (attributeset_is_avail(var->Attributes, ATTR_ITEM))
		fill_list_by_items(var->Attributes, var->Widget);

	if (attributeset_cmp_left
	    (var->Attributes, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(var->Widget, FALSE);

	gtk_widget_queue_draw(var->Widget);
}

void widget_table_refresh(variable * var) {
	char *act;
	if (var == NULL || var->Attributes == NULL)
		return;
		
	/// The <input> tags
	act = attributeset_get_first(var->Attributes, ATTR_INPUT);
	while (act != NULL) {
		if (strncasecmp(act, "command:", 8) == 0)
			fill_table_by_command(var->Widget, act + 8);
		act = attributeset_get_next(var->Attributes, ATTR_INPUT);
	}

	if (attributeset_is_avail(var->Attributes, ATTR_ITEM))
		fill_clist_by_items(var->Attributes, var->Widget, '|');

	if (attributeset_cmp_left
	    (var->Attributes, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(var->Widget, FALSE);
}

void widget_combo_refresh(variable * var) {
	if (var == NULL || var->Attributes == NULL)
		return;
		
	/// The <item> tags
	if (attributeset_is_avail(var->Attributes, ATTR_ITEM))
		fill_combo_by_items(var->Attributes, var->Widget);

	if (attributeset_cmp_left
	    (var->Attributes, ATTR_VISIBLE, "disabled"))
		gtk_widget_set_sensitive(var->Widget, FALSE);
}

extern char *includefile;

FILE *widget_opencommand(const char *command) {
	char *the_line;
	FILE *infile;

	if (includefile != NULL) {
		the_line = g_malloc(strlen(command) + strlen(includefile) + 16);
		sprintf(the_line, ". %s; %s", includefile, command);	//technosaurous
		infile = popen(the_line, "r");
	} else {
		infile = popen(command, "r");
	}
	if (infile == NULL) {
		fprintf(stderr, "%s(): %m\n", __func__);
		fflush(stderr);
	}
	return (infile);
}

void fill_label_by_command(GtkWidget *label, char *command) {
	FILE *infile;
	char oneline[512];
	gchar *error_msg;
	GString *text = g_string_sized_new(512);

	infile = widget_opencommand(command);
	if (infile == NULL) {
		g_warning("%s(): command %s, %m\n", __func__, command);
		error_msg = g_strdup_printf("Unable to start command: %s", command);
		gtk_label_set_text(GTK_LABEL(label), error_msg);
		g_free(error_msg);
		return;
	}

	while (fgets(oneline, 512, infile) != NULL) 
		g_string_append(text, oneline);

ready_reading:
	gtk_label_set_text(GTK_LABEL(label), text->str);
	g_string_free(text, TRUE);
	pclose(infile);
}

void fill_clist_by_command(GtkWidget * list, int columns, char *command) {
	FILE *infile;
	char *oneline = NULL;
	list_t *thisline;
	uint size = 0;
	
	/// Opening pipe for reading
	infile = widget_opencommand(command);
	if (infile == NULL) {
		fprintf(stderr, "%s(): command %s, %m\n", __func__,	command);
		return;
	}

	/// Reading and filling the lines to the clist
	while (getline(&oneline, &size, infile) != -1) {
		thisline = linecutter(oneline, '|');
		gtk_clist_append(GTK_CLIST(list), thisline->line);
	}

	pclose(infile);
}

void fill_list_by_command(GtkWidget * list, char *command) {
	FILE *infile;
	char oneline[512];
	char *saved;
	GtkWidget *item;

	infile = widget_opencommand(command);
	if (infile == NULL) {
		fprintf(stderr, "%s(): command %s, %m\n", __func__,	command);
		return;
	}
	while (fgets(oneline, 512, infile) != NULL) {
		if (oneline[strlen(oneline) - 1] == '\n')
			oneline[strlen(oneline) - 1] = '\0';
		/* Who the hell will free this memory? */
		saved = strdup(oneline);
		item = gtk_list_item_new_with_label(oneline);
		gtk_widget_show(item);
		gtk_object_set_user_data(GTK_OBJECT(item),
					 (gpointer) saved);
		gtk_container_add(GTK_CONTAINER(list), item);
	}
	pclose(infile);
}

void fill_entry_by_command(GtkWidget * entry, char *command) {
	FILE *infile;
	char line[512];

	g_assert(entry != NULL && command != NULL);

	infile = widget_opencommand(command);
	if (infile == NULL) {
		fprintf(stderr, "%s(): command %s, %m\n", __func__,	command);
		return;
	}

	if (fgets(line, 512, infile) != NULL) {
		if (line[strlen(line) - 1] == '\n')
			line[strlen(line) - 1] = '\0';
		gtk_entry_set_text(GTK_ENTRY(entry), (const gchar *) line);
	}

	pclose(infile);
}

void fill_checkbox_by_command(GtkWidget *checkbox, char *command) {
	FILE *infile;
	char line[512];

	g_assert(checkbox != NULL && command != NULL);

	infile = widget_opencommand(command);
	if (infile == NULL) {
		fprintf(stderr, "%s(): command %s, %m\n", __func__, command);
		return;
	}

	if (fgets(line, 512, infile) != NULL) {
		if (line[strlen(line) - 1] == '\n')
			line[strlen(line) - 1] = '\0';
		if (strcmp(line, "true") == 0 || strcmp(line, "yes") == 0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
		else
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), FALSE);
	}
	pclose(infile);
}

void fill_table_by_command(GtkWidget * list, char *command) {
	FILE *infile;
	char oneline[512];
	int q, n, length;
	char *c[32];
	infile = widget_opencommand(command);
	if (infile == NULL) {
		fprintf(stderr, "%s(): command %s, %m\n", __func__,	command);
		return;
	}

	while (fgets(oneline, 512, infile) != NULL) {
		if (oneline[strlen(oneline) - 1] == '\n')
			oneline[strlen(oneline) - 1] = '\0';
		for (q = 0; q < 32; ++q)
			c[q] = NULL;

		c[0] = oneline;
		n = 1;
		length = strlen(oneline);
		for (q = 0; q < length; ++q) {
			if (oneline[q] == '|') {
				oneline[q] = '\0';
				c[n] = &oneline[q + 1];
				n++;
			}
		}
		gtk_clist_append((GtkCList *) list, c);
	}
	pclose(infile);
}

char *widgets_to_str(int itype) {
	char *type;
	switch (itype) {
	case WIDGET_LABEL:
		type = "LABEL";
		break;
	case WIDGET_ENTRY:
		type = "ENTRY";
		break;
	case WIDGET_EDIT:
		type = "EDIT";
		break;
	case WIDGET_BUTTON:
		type = "BUTTON";
		break;
	case WIDGET_CHECKBOX:
		type = "CHECKBOX";
		break;
	case WIDGET_RADIO:
		type = "RADIO";
		break;
	case WIDGET_LIST:
		type = "LIST";
		break;
	case WIDGET_TABLE:
		type = "TABLE";
		break;
	case WIDGET_COMBO:
		type = "COMBO";
		break;
	case WIDGET_OKBUTTON:
		type = "OKBUTTON";
		break;
	case WIDGET_CANCELBUTTON:
		type = "CANCELBUTTON";
		break;
	case WIDGET_HELPBUTTON:
		type = "HELPBUTTON";
		break;
	case WIDGET_NOBUTTON:
		type = "NOBUTTON";
		break;
	case WIDGET_YESBUTTON:
		type = "YESBUTTON";
		break;
	case WIDGET_SCROLLEDW:
		type = "SCROLLEDW";
		break;
	case WIDGET_VBOX:
		type = "VBOX";
		break;
	case WIDGET_HBOX:
		type = "HBOX";
		break;
	case WIDGET_FRAME:
		type = "FRAME";
		break;
	case WIDGET_WINDOW:
		type = "WINDOW";
		break;
	case WIDGET_PIXMAP:
		type = "PIXMAP";
		break;
	case WIDGET_MENUBAR:
		type = "MENUBAR";
		break;
	default:
		type = "THINGY";
	}
	return (type);
}
